import cx from 'classnames';
import PropTypes from 'prop-types';
import React, { forwardRef, useState } from 'react';
import TypeaheadManager from '../../core/TypeaheadManager';
import useRootClose from '../../core/useRootClose';
import useTypeahead from '../../core/useTypeahead';
import ClearButton from '../ClearButton';
import Loader from '../Loader';
import { useOverlay } from '../Overlay';
import Token from '../Token/Token';
import TypeaheadInputMulti from '../TypeaheadInputMulti';
import TypeaheadInputSingle from '../TypeaheadInputSingle';
import TypeaheadMenu from '../TypeaheadMenu';
import { getOptionLabel, isFunction, isSizeLarge, preventInputBlur, } from '../../utils';
import { checkPropType, inputPropsType, sizeType } from '../../propTypes';
const propTypes = {
    clearButton: PropTypes.bool,
    inputProps: checkPropType(PropTypes.object, inputPropsType),
    isInvalid: PropTypes.bool,
    isLoading: PropTypes.bool,
    isValid: PropTypes.bool,
    renderInput: PropTypes.func,
    renderMenu: PropTypes.func,
    renderToken: PropTypes.func,
    size: sizeType,
};
const defaultRenderMenu = (results, menuProps, props) => (React.createElement(TypeaheadMenu, { ...menuProps, labelKey: props.labelKey, options: results, text: props.text }));
const defaultRenderToken = (option, props, idx) => (React.createElement(Token, { disabled: props.disabled, key: idx, onRemove: props.onRemove, option: option, tabIndex: props.tabIndex }, getOptionLabel(option, props.labelKey)));
function Typeahead(props) {
    const { children, clearButton, disabled, emptyLabel, isLoading, id, isInvalid, isValid, maxHeight, multiple, newSelectionPrefix, paginationText, renderMenuItemChildren, results, size, } = props;
    const rootElementRef = useRootClose(props.hideMenu, {
        disabled: props.open || !props.isMenuShown,
    });
    const [referenceElement, setReferenceElement] = useState(null);
    const overlayProps = useOverlay(referenceElement, props);
    const renderMenu = props.renderMenu || defaultRenderMenu;
    const menuProps = {
        ...overlayProps,
        emptyLabel,
        id,
        maxHeight,
        newSelectionPrefix,
        paginationText,
        renderMenuItemChildren,
    };
    function renderInput() {
        const inputProps = {
            ...props.getInputProps(props.inputProps),
            referenceElementRef: setReferenceElement,
        };
        if (props.renderInput) {
            return props.renderInput(inputProps, props);
        }
        const commonProps = {
            ...inputProps,
            isInvalid,
            isValid,
            size,
        };
        if (!multiple) {
            return React.createElement(TypeaheadInputSingle, { ...commonProps });
        }
        const { labelKey, onRemove, selected } = props;
        const renderToken = props.renderToken || defaultRenderToken;
        const tokenProps = { ...commonProps, labelKey, onRemove };
        return (React.createElement(TypeaheadInputMulti, { ...commonProps, placeholder: selected.length ? '' : inputProps.placeholder, selected: selected }, selected.map((option, idx) => renderToken(option, tokenProps, idx))));
    }
    let auxContent;
    if (isLoading) {
        auxContent = React.createElement(Loader, null);
    }
    else if (clearButton && !disabled && props.selected.length) {
        auxContent = (React.createElement(ClearButton, { onClick: props.onClear, onMouseDown: preventInputBlur, size: size }));
    }
    return (React.createElement("div", { className: cx('rbt', {
            'has-aux': !!auxContent,
            'is-invalid': props.isInvalid,
            'is-valid': props.isValid,
        }, props.className), ref: rootElementRef, style: {
            ...props.style,
            outline: 'none',
            position: 'relative',
        }, tabIndex: -1 },
        renderInput(),
        props.isMenuShown && renderMenu(results, menuProps, props),
        auxContent && (React.createElement("div", { className: cx('rbt-aux', { 'rbt-aux-lg': isSizeLarge(size) }) }, auxContent)),
        isFunction(children) ? children(props) : children));
}
Typeahead.propTypes = propTypes;
const TypeaheadExport = forwardRef((props, ref) => {
    const typeaheadProps = useTypeahead(props, ref);
    return (React.createElement(TypeaheadManager, { ...typeaheadProps }, (managerProps) => React.createElement(Typeahead, { ...props, ...managerProps })));
});
export default TypeaheadExport;
